#!/bin/bash

PROJECT="NGFW"

required_packages="sudo gnupg curl sshpass"
app_deps_packages="snmp snmptrapd nginx libssh-4"
backports_packages="suricata"

script="$(basename "$0")"
BASE_DIR=$(dirname "$0")

. "${BASE_DIR}/common.sh" \
    || { echo "ERROR: Unable to source ${BASE_DIR}/common.sh" >&2; exit 1; }

cd "${BASE_DIR}" \
    || fail "Unable to enter installer directory"


help() {
	local app="$1"

	cat <<-END
		Usage:
		  ${app} [options]

		Options:
		  --help, -h        - get this help
		  --install, -i     - install ${PROJECT} (default)
		  --uninstall, -u   - uninstall ${PROJECT}
	END
}

install_mongo() {
	rm -f /usr/share/keyrings/mongodb-server-7.0.gpg
	curl -fsSL https://www.mongodb.org/static/pgp/server-7.0.asc \
	    | gpg -o /usr/share/keyrings/mongodb-server-7.0.gpg --dearmor
	[ "$?" -eq 0 ] \
	    || fail "Can't install mongo repository key"

	echo "deb [ signed-by=/usr/share/keyrings/mongodb-server-7.0.gpg ] http://repo.mongodb.org/apt/debian bookworm/mongodb-org/7.0 main" \
	    | tee /etc/apt/sources.list.d/mongodb-org-7.0.list
	[ "$?" -eq 0 ] \
	    || fail "Can't add mongo repository"

	apt update \
	    && apt install -y mongodb-org \
	    || fail "Can't install mongo server"

	log_msg "Start mongo ..."
	systemctl daemon-reload \
	    && systemctl enable mongod \
	    && systemctl start mongod \
	    || fail "Can't start mongo server"
}

install_backports_packages() {
	local found

	found=$(cat /etc/apt/sources.list /etc/apt/sources.list.d/* | grep "^deb[ \t].*[ \t]bookworm-backports[ \t]*main")
	if [ -z "${found}" ]; then
		log_msg "Adding debian backports repository ..."
		echo "deb http://deb.debian.org/debian/ bookworm-backports main" > "/etc/apt/sources.list.d/bookworm-backports.list"

		apt update \
		    || fail "Can't update package database, check your internet connection."
	fi

	apt install -t bookworm-backports -y "$@" \
	    || fail "Can't install required packages"
}

install_app() {
	log_msg "Installing ${PROJECT} on ${ID} ${VERSION_CODENAME} for ${arch} ..."

	log_msg "Updating package database ..."
	apt update \
	    || fail "Can't update package database, check your internet connection."

	log_msg "Trying to fix broken deps ..."
	apt -f install -y \
	    || fail "Unable to fix broken deps"

	log_msg "Install necesary packages ..."
	apt install -y sudo ${required_packages} ${app_deps_packages} \
	    || fail "Can't install required packages"

	log_msg "Install backports packages ..."
	install_backports_packages ${backports_packages}

	log_msg "Install mongo ..."
	install_mongo

	# this step may fail (some deps may not be installed yet)
	log_msg "Install ${PROJECT} packages ..."
	dpkg -i \
		pkg/libyang3_*_${deb_arch}.deb \
		pkg/libyang-tools_*_${deb_arch}.deb \
		pkg/libyang-modules_*_all.deb \
		pkg/libnetconf4_*_${deb_arch}.deb \
		pkg/libsysrepo7_*_${deb_arch}.deb \
		pkg/sysrepo-tools_*_${deb_arch}.deb
	dpkg -i \
		pkg/netopeer2_*_${deb_arch}.deb

	log_msg "Fixing broken deps ..."
	apt -f install -y \
	    || fail "Unable to fix broken deps"

	log_msg "Install isabelle ..."
	rm -rf /opt/isabelle
	mkdir -p /opt/isabelle \
	    && tar -C /opt -xf "pkg/isabelle-core.${arch}.tar.gz" \
	    && tar -C /opt -xf "pkg/isabelle-core-manage.${arch}.tar.gz" \
	    && tar -C /opt -xf "pkg/isabelle-ui-manage.all.tar.gz" \
	    || fail "Unable to install isabelle"

	# setup host
	log_msg "Install NETCONF keys ..."
	cp -rt /opt/isabelle keys \
	    && chown -R root:root /opt/isabelle/keys \
	    && chmod 600 /opt/isabelle/keys/* \
	    || fail "Unable to install NETCONF keys"

	# setup netopeer2
	log_msg "Install netopeer2 keys ..."
	netopeer2-cli <<-END
		auth keys add /opt/isabelle/keys/netconf.pub /opt/isabelle/keys/netconf
		auth pref publickey 1
		auth pref password 0
		auth pref interactive 0
	END

	log_msg "Setup nginx & isabelle ..."
	cp -rt / etc \
	    || fail "Unable to update system configuration"

	# remove default nginx config (it conflicts with our config)
	rm -f /etc/nginx/sites-enabled/default

	log_msg "Creating initial mongo database ..."
	/opt/isabelle/run-core.sh --first-run \
	    || fail "Unable to start process to create initial mongo database"

	log_msg "Reloading systemctl daemon configuration ..."
	systemctl daemon-reload

	log_msg "Start snmptrapd ..."
	systemctl start snmptrapd

	log_msg "Start isabelle-core ..."
	systemctl enable isabelle-core \
	    && systemctl start isabelle-core \
	    || fail "Unable to start isabelle-core"

	log_msg "Start nginx ..."
	systemctl enable nginx \
	    && systemctl restart nginx \
	    || fail "Unable to start nginx"

	log_msg "Installation: OK"
}

uninstall_app() {
	log_msg "Uninstalling ${PROJECT} ..."

	log_msg "Stopping snmptrapd, nginx, isabelle-core, mongod ..."
	systemctl stop snmptrapd
	systemctl stop nginx
	systemctl stop isabelle-core
	systemctl stop mongod

	log_msg "Remove nginx & isabelle-core files ..."
	rm -rf \
		/root/.netopeer2-cli \
		/etc/nginx/sites-available/isabelle.conf \
		/etc/nginx/sites-enabled/isabelle.conf \
		/etc/systemd/system/isabelle-core.service \
		/etc/selfcerts \
		/opt/isabelle

	log_msg "Restore original nginx config ..."
	ln -s ../sites-available/default \
		/etc/nginx/sites-enabled/default 2>/dev/null

	log_msg "Reloading systemctl daemon configuration ..."
	systemctl daemon-reload

	log_msg "Removing ${PROJECT} packages ..."
	apt remove -y ${app_deps_packages} ${backports_packages} \
		netopeer2 sysrepo-tools libsysrepo7 libnetconf4 libyang-modules libyang-tools libyang3

	log_msg "Removing mongo ..."
	apt purge -y "mongodb-org*"

	log_msg "Cleanup package database ..."
	apt autoremove -y
	apt -f install

	# purge package configuration
	dpkg -P $(dpkg -l | grep ^r | awk '{print $2}')

	log_msg "Removing mongo databases & logs ..."
	rm -rf \
		/var/lib/mongodb \
		/var/log/mongodb

	log_msg "Uninstallation: OK"
}


mode="install_app"
while [ -n "$1" ]; do
	opt="$1"
	shift
	case "${opt}" in
		--help|-h)
			help "${script}"
			exit
			;;
		--install|-i)
			mode="install_app"
			;;
		--uninstall|-u)
			mode="uninstall_app"
			;;
		*)
			echo "Unknown option ${opt}"
			help "${script}"
			exit 1
			;;
	esac
done

arch=$(uname -m)
deb_arch=$(debian_arch ${arch})

ls pkg/*_${deb_arch}.deb >/dev/null 2>&1 \
    || fail "This installer is NOT for your architecture."

[ -e "/etc/os-release" ] \
    || fail "Unknown OS, unable to continue"

. "/etc/os-release"

case "${ID}" in
	debian|ubuntu) ;;
	*)	fail "${ID} is unsupported"; ;;
esac

case "${VERSION_CODENAME}" in
	bookworm|trixie|noble) ;;
	*)	fail "${ID} is unsupported"; ;;
esac

id=$(id -u)
[ "$id" -eq 0 ] \
    || fail "Unable to continue, must be run as root"

# install/uninstall app
"$mode"
